/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.kenai.com
 * SCM: https://kenai.com/hg/openbluesky~src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.annotation.Nonnull;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import it.tidalwave.util.As;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.role.ui.PresentationModel;
import org.openide.util.Parameters;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;

/***********************************************************************************************************************
 *
 * A specialization of {@link GenericFilterNode} which filters according to a certain object contained in the
 * {@link Node}, delegating to an {@link ObjectFilter}.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public final class FilteredNodePresentationModel<T> extends GenericFilterNode implements PresentationModel
  {
    static class ProxyChildren<T> extends FilterNode.Children
      {
        @Nonnull
        private final ObjectFilter objectFilter;

        @Nonnull
        private final Class<?> objectClass;
        
        public ProxyChildren (final @Nonnull Node parent,
                              final @Nonnull ObjectFilter<T> objectFilter,
                              final @Nonnull Class<T> objectClass)
          {
            super(parent);
            Parameters.notNull("parent", parent);
            Parameters.notNull("objectFilter", objectFilter);
            Parameters.notNull("objectClass", objectClass);
            this.objectFilter = objectFilter;
            this.objectClass  = objectClass;
          }

        @Override @Nonnull
        protected Node copyNode (final @Nonnull Node delegate)
          {
            return new FilteredNodePresentationModel(delegate, objectFilter, objectClass);
          }

        @Override @Nonnull
        protected Node[] createNodes (final @Nonnull Node parentNode)
          {
            final List<Node> result = new ArrayList<Node>();

            for (final Node childNode : super.createNodes(parentNode))
              {
                if (objectFilter.accept(childNode.getLookup().lookup(objectClass)))
                  {
                    result.add(childNode);
                  }
              }

            return result.toArray(new Node[result.size()]);
          }
      }

    public FilteredNodePresentationModel (final @Nonnull Node delegate,
                                final @Nonnull ObjectFilter objectFilter,
                                final @Nonnull Class<T> objectClass)
      {
        super(delegate, new ProxyChildren<T>(delegate, objectFilter, objectClass));
      }
    
   /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public final <T> T as (final @Nonnull Class<T> type)
      {
        return as(type, As.Defaults.throwAsException(type));
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    public <T> T as (final @Nonnull Class<T> type, final @Nonnull NotFoundBehaviour<T> notFoundBehaviour)
      {
        final T result = getLookup().lookup(type);

        return (result != null) ? result : notFoundBehaviour.run(new NotFoundException(type.getName()));
      }
    
    @Override
    public void addPropertyChangeListener (String string, PropertyChangeListener pl)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public void removePropertyChangeListener (String string, PropertyChangeListener pl) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public boolean hasListeners (String string)  
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public PropertyChangeListener[] getPropertyChangeListeners() 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Override
    public PropertyChangeListener[] getPropertyChangeListeners(String string) 
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }

